/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * ARGS.H  Argument handling functions for -o, -i and -l switches
 *         -x=format,arg1,arg2,arg3
 *         -x,arg1,arg2
 */

#ifndef _CAS_ARGS_H_
#define _CAS_ARGS_H_

/*
 * the list of arguments is stored in a linked list rather than an array
 * to facilitate the editing of arg lists and insertion of arg macros
 */

typedef struct darg_s {
  arg_t *defargs;
  arg_t *newargs;
} darg_t;

#define ARGS_DEF_UNDEFINED	0
#define ARGS_DEF_IN     1
#define ARGS_DEF_OUT    2
#define ARGS_DEF_LIST   3

arg_t  *args_split(char *argstr, arg_t *endp);
char   *args_get_first(arg_t **args);
void    args_join(arg_t *new, int whicharr, int index, darg_t *dargs);
void    args_load_defaults(void);

int     args_is_set(darg_t dargs, ...);
char   *args_get_string(darg_t dargs, ...);
long    args_get_long(darg_t dargs, long def, ...);
char    args_get_char(darg_t dargs, char def, ...);
int     args_is_set_keyed(darg_t dargs, char *key, ...);

#ifdef _CAS_ARGS_C_

#define ARGS_RC_LINELEN			1024

typedef struct macro_s {
  char *name;
  arg_t *args;
  struct macro_s *next;
} macro_t;

typedef struct defaults_s {
  arg_t *in[CAS_NUM_FORMATS];       /* indexed with types/formats enums */
  arg_t *out[CAS_NUM_FORMATS];
  arg_t *list[CAS_NUM_TYPES];
  macro_t *macro;
  /* conversions also needed here */
} defaults_t;

static defaults_t *defaults;

static void args_free(arg_t *args);
static arg_t *get_arg_match(char *name, arg_t *args);
static arg_t *get_darg_match(darg_t dargs, va_list va);
static void add_setting(char *name, char *data);
static void add_arg(char *iol, char *type, char *data);
static void add_macro(char *name, char *data);
static int args_expand_macro(char *name, arg_t **args);

#endif /* _CAS_ARGS_C_ */

#endif /* !_CAS_ARGS_H_ */

/* end of args.h */
