/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * CAS.H  standard includes and defines.  should be included by all .c files
 */

#ifndef CAS_H
#define CAS_H

#define CAS

/* standard #includes */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h> 
#include <string.h>
#include <stdarg.h>

/* include configuration */

#include "config.h"

/* Versioning */

#define CAS_VER_NUM         CAS_MAJOR_VER "." CAS_MINOR_VER
#define CAS_VER_TEMP         CAS_VER_OS " version " CAS_VER_NUM
#ifdef CAS_COMM
# define CAS_VER_STR    CAS_VER_TEMP
#else
# define CAS_VER_STR    CAS_VER_TEMP " (no-comms)"
#endif

#define TRUE 				1
#define FALSE				0

#define EXIT_BUG            0   /* Bug / unimplemented feature */
#define EXIT_CMD            1   /* Error in Command Line */
#define EXIT_FILE           2   /* File access error */
#define EXIT_MEM            3   /* Unable to allocate memory */
#define EXIT_COMM           4   /* Communications error */
#define EXIT_FORMAT         5   /* Error in file format */

#define MDL_UNKNOWN              0xfe
#define MDL_NOTGEN               0x01   /* flag */
#define MDL_7700                 0x02
#define MDL_8700                 MDL_7700
#define MDL_9700                 0x04
#define MDL_9800                 0x08
#define MDL_9850                 0x10
#define MDL_9950                 MDL_9850
#define MDL_9750                 MDL_9850

#define SUBF_UNKNOWN    '\0'
#define SUBF_9700		'9'
#define SUBF_9850       '5'
#define SUBF_RAW		'R'

/*
 * All data structures in cas.h refer to the abstract forms of data stored,
 * and so only correspond with file formats where the storage methods
 * coincide.  File-format specific structures are in the relevant fXXX.h
 * (or sXXXX.h for .CAS / comms structures/values)
 */

/* Formats and Types */

/*
 * VERY IMPORTANT!  [sd]_types and [sd]_formats *must* be kept up-to-date
 * with respect to the types and formats enumerations.  Never rely on the
 * positions of items in these enums / arrays.
 */
 
typedef enum {fUnknown,fDefault,fCTF,fCAS,fFXP,fBMP,fGIF,fCOM,fBIN} formats;

#define CAS_NUM_FORMATS         9

#ifdef CAS_DEFINE_GLOBALS
char *s_formats[CAS_NUM_FORMATS] = {
  "Unknown","Default","CTF","CAS","FXP","BMP","GIF","COM","BIN"
};

char *d_formats[CAS_NUM_FORMATS] = {
  "Unknown","Default",
  "Calculator Text Format",
  "Casio's binary format",
  "FX Program",
  "Windows bitmap",
  "Graphics Interchange Format",
  "Serial communication",
  "Binary file"
};
#else /* CAS_DEFINE_GLOBALS */
extern char *s_formats[CAS_NUM_FORMATS];
extern char *d_formats[CAS_NUM_FORMATS];
#endif /* CAS_DEFINE_GLOBALS */


typedef enum {
  tUnknown=0,tAll,             /* tAll used by conversions *only* */
  tOldProg,tEditor,tFn,tSSMono,tSSCol,
  tVarMem,tDefMem,tAllMem,tSD,tLR,tMatrix,tRecTab,tFnTab,
  tPoly,tSimul,tZoom,tDyna,tGraphs,tRange,
  tBackup,tEnd,tRaw,tText,tDesc
} types;

#define CAS_NUM_TYPES           26

#ifdef CAS_DEFINE_GLOBALS
char *s_types[CAS_NUM_TYPES] = {
  "Unknown","All",
  "OldProg","Editor","Fn","SSMono","SSCol",
  "VarMem","DefMem","AllMem","SD","LR","Matrix","RecTab","FnTab",
  "Poly","Simul","Zoom","Dyna","Graphs","Range",
  "Backup","End","Raw","Text","Desc"
};
char *d_types[CAS_NUM_TYPES] = {
  "Unknown", "any type",
  "unnamed program","named program or editor file","F-Memory",
  "monochrome screenshot",
#ifdef PROPER_SPELLING
  "colour screenshot",
#else
  "color screenshot",
#endif
  "variable memories","defined memories","all memories",
  "single variable data","paired variable data","matrix",
  "recursion table","function table",
  "polynomial equation","simultaneous equations"
  "graph zoom","dynagraph","graphs","graph range",
  "backup","end marker","raw binary data","raw text","description"
};

#else	/* CAS_DEFINE_GLOBALS */
extern char *s_types[CAS_NUM_TYPES];
extern char *d_types[CAS_NUM_TYPES];
#endif	/* CAS_DEFINE_GLOBALS */

typedef struct arg_s {
  char *name;
  char *value;
  int set;          /* set to 0 to mean _unset_ ie no-{name} */
  struct arg_s *next;
} arg_t;

typedef struct convs {
  types from;
  types to;
  int after;
  arg_t *args;
  struct convs *next;
} convs;

typedef struct flagtype {
  struct {
    formats format;
    char *filename;
    int subf;
    arg_t *args;
  } input;
  struct {
    formats format;
    char *filename;
    int subf;
    arg_t *args;
  } output;
  struct {
    int what;
    arg_t *args;
  } list;
  convs *convert;
  int model;
  int verbose;
  int pager;
  FILE *debug;
  void *castle;
} flagtype;

#ifdef CAS_DEFINE_GLOBALS       /* so there is only one definition */

char *argv0;

unsigned char read_csum;
unsigned char read_colon;

unsigned char pow2[8]={1,2,4,8,0x10,0x20,0x40,0x80};

flagtype flags={
  {fUnknown,NULL,SUBF_UNKNOWN,NULL},        /* input */
  {fUnknown,NULL,SUBF_UNKNOWN,NULL},        /* output */
  {0,NULL},                                 /* list */
  NULL,                                     /* conversions */
  MDL_UNKNOWN,                              /* model */
  FALSE,                                    /* verbose */
  FALSE,                                    /* pager */
  NULL,                                     /* debug */
  NULL                                      /* communication with castle */
};

char *ProgramName[]={"0","1","2","3","4","5","6","7","8","9",
                     "A","B","C","D","E","F","G","H","I","J",
                     "K","L","M","N","O","P","Q","R","S","T",
                     "U","V","W","X","Y","Z","\\r","\\th"};

#else

extern char *argv0;
extern unsigned char read_csum;
extern unsigned char read_colon;
extern unsigned char pow2[8];
extern flagtype flags;
extern char *ProgramName[];

#endif /* CAS_DEFINE_GLOBALS */

/* Information structure */

typedef struct casinfo {
  types type;                   /* type of data in this block */
  void *data;                   /* Pointer to beginning of data */
  size_t length;                /* Actual length of data */
  struct casinfo *next;         /* pointer to next item. */

  /* type specific stuff */

  union {   /* type specific info */
    struct {
      int progtype;
      int prognum;      /* 0 to 37 */
    } prg; /* old-style program */
    struct {
      unsigned char filename[13];
      unsigned char password[13];
    } ed;  /* editor file / new-style program */
    struct {
      int height;
      int width;
      int bpp;
      int order;
    } ss; /* screenshot */
    struct {
      int cooked;
    } raw; /* raw data */
  } s;
} casinfo;

#define SSOFFSET(i,x,y)     ((x)+(y)*((i)->s.ss.width))
#define SSDATA(i,x,y) (((unsigned char *)((i)->data))[(x)+(y)*(i)->s.ss.width])

/* As CaS stores the program type in the same way to the .CAS file, these
   defines are here and not in s9700.h */

#define CAS_PT_SSTO             0x02
#define CAS_PT_MATRIX           0x04
#define CAS_PT_SD               0x10
#define CAS_PT_LR               0x20
#define CAS_PT_BASEN            0x40
#define CAS_PT_SDRAW            0x80

/* structure of read/write functions used for .CAS files and communications */

typedef struct iostruct {
  int (* readblk)(void *, size_t);
  int (* writeblk)(void *, size_t);
  size_t (* readraw)(void *, int);
  int (* writeraw)(void *, size_t);
} iostruct;

#endif /* CAS_H */

/* end of cas.h */
