/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * CASTLE.C  Routines for interfacing with the Castle IDE
 */

/*
 * the actual communications mechanism is abstracted here, so that on
 * different (superior) OS's, castle could be informed perhaps in real
 * time or in any of many different ways.
 */

#ifndef NO_CASTLE

#define _CAS_CASTLE_C_

#include "cas.h"
#include "misc.h"
#include "castle.h"

/* interface routines */

void castle_raw(castle_code type, const char *fmt,...)
{
  va_list args;

  va_start (args,fmt);
  castle_send(type,fmt,args);
  va_end (args);
}

void castle_comment(const char *fmt,...)
{
  va_list args;

  va_start (args,fmt);
  castle_send(CASTLE_COMMENT,fmt,args);
  va_end (args);
}

void castle_format(castle_code base, formats format, char *filename)
{
  switch (format) {
    case fCTF: base+=CASTLE_FMT_CTF;break;
    case fCAS: base+=CASTLE_FMT_CAS;break;
    case fFXP: base+=CASTLE_FMT_FXP;break;
    case fBMP: base+=CASTLE_FMT_BMP;break;
    case fGIF: base+=CASTLE_FMT_GIF;break;
    case fCOM: base+=CASTLE_FMT_COM;break;
    case fBIN: base+=CASTLE_FMT_BIN;break;
    default:;
  }

  castle_raw(base, "%s\n", filename);
}


/* functions to control and use communications channel */

void castle_open(char *argw)
{
  argw+=2;if (*argw=='='||*argw==':') argw++;

  if (*argw)
    flags.castle=(FILE *)tsys_fopen(argw,"w",FALSE);
  else
    Msg(EXIT_CMD,"main","No file specified for -e switch");

  if (!flags.castle)
    Msg(EXIT_CMD,"main","unable to open file `%s' for -e switch",argw);
}

void castle_close(void)
{
  tsys_fclose((FILE *)flags.castle);
}

/* function to send data to castle */

void castle_send(castle_code type, const char *fmt, va_list args)
{
  static castle_code lastcode = 0;
  static int midway=0;
  char buf[CASTLE_BUFSIZE];
  register char *i;

  if (!flags.castle) return;

  if (midway && lastcode != type) {
    putc('\n', (FILE *)flags.castle);
    midway=0;
  }
  lastcode=type;

  vsprintf(buf, fmt, args);     /* vsnprintf should be standard! */
  for (i=buf; *i; i++) {
    if (!midway) {
      fprintf((FILE *)flags.castle,"%05d ",type);
      midway=1;
    }
    if (*i=='\n') {
      putc('\n',(FILE *)flags.castle);
      midway=0;
    } else {
      putc(*i,(FILE *)flags.castle);
    }
  }
}

#endif /* NO_CASTLE */

/* end of castle.c */

