/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * CDJGPP.C  Communication routines for use under DJGPP.
 */
 
#include "cas.h"
#include "misc.h"
#include "svasync.h"
#include "commsys.h"

#define COMM_TIMEOUT          500000L     /* 500,000 */

int COMsysInit(char *filename, int baud, char parity, int handshake)
{
  int comport=-1;
  int parnum=EVEN_PARITY;
  int hand=0;

  if (!stricmp(filename,"COM1")) comport=COM1;
  if (!stricmp(filename,"COM2")) comport=COM2;
  if (!stricmp(filename,"COM3")) comport=COM3;
  if (!stricmp(filename,"COM4")) comport=COM4;
  if (comport==-1) return FALSE;

  switch (parity) {
    case CASPORT_NONE: parnum=NO_PARITY;break;
    case CASPORT_ODD: parnum=ODD_PARITY;break;
    case CASPORT_EVEN: ;
    default: parnum=EVEN_PARITY;break;
  }

  if (handshake&CASPORT_DTR) hand |=DTR;
  if (handshake&CASPORT_RTS) hand |=RTS;

  /* Initialise COM port */

  if (SVAsyncInit(comport))
    return FALSE;

  atexit(SVAsyncStop);  /* cleans up COM port upon exit */

  /* Initialise the FIFO buffer on the UART */

#if 0
  switch (SVAsyncFifoInit()) {
    case 0: Msg(M_VERBOSE,"com","16550 FIFO or better detected");break;
    case 1: Msg(M_VERBOSE,"com","16450 (or 16550 no FIFO) detected");break;
    case 2: Msg(M_VERBOSE,"com","8250 or below detected");break;
    default: Msg(M_VERBOSE,"com","Unknown UART detected");
  }
#endif

  /* set baud rate and parity.  Calc uses 8 data bits + 2 stop bits */

  SVAsyncSet(baud,BITS_8|parnum|STOP_2);

  /* raise power lines */

  if (hand) SVAsyncHand(hand);

  return TRUE;
}

void COMsysClose(void)
{
  SVAsyncStop();
}

void COMsysPutChar(unsigned char data)
{
  SVAsyncOut(data);
  if (flg_bytes)
    printf("{%02x}",data);fflush(stdout);
}

unsigned char COMsysGetChar(void)
{
  unsigned char rdchar;

  for(;;) {
    if (SVAsyncInStat())
      if (flg_bytes) {
        rdchar=SVAsyncIn();
        printf("[%02x]",rdchar);fflush(stdout);
        return rdchar;
      } else {
        return SVAsyncIn();
      }
  }
  /* Msg(EXIT_COMM,"comm/read","Timeout reading from COM port"); */
  return 0;
}

unsigned char COMsysGetCharTimeout(int *success_r,unsigned int modifier)
{
  unsigned long x;
  unsigned long timeout;
  unsigned char rdchar;

  timeout=COMM_TIMEOUT * 100 / modifier;

  for(x=0;x<timeout;x++) {
    if (SVAsyncInStat()) {
      *success_r=TRUE;
      if (flg_bytes) {
        rdchar=SVAsyncIn();
        printf("[%02x]",rdchar);fflush(stdout);
        return rdchar;
      } else {
        return SVAsyncIn();
      }
    }
  }
  *success_r=FALSE;
  return 1;
}


/* end of cdjgpp.c */
