/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * CTURBOC.C  Communications routines for use with Turbo C
 */

#if 1
/* seems to compile the ASM with upper case ? :-| */
# define ASyncInit(x)        ASYNCINIT((x))
# define ASyncSet(x,y)       ASYNCSET((x),(y))
# define ASyncHand(x)        ASYNCHAND((x))
# define ASyncStop()         ASYNCSTOP()
# define ASyncInStat()       ASYNCINSTAT()
# define ASyncIn()           ASYNCIN()
# define ASyncOut(x)         ASYNCOUT((x))
#endif

#include <conio.h>

#include "cas.h"
#include "misc.h"
#include "async.h"
#include "commsys.h"

#define COMM_TIMEOUT        1000000L     /* 1,000,000 */

extern flagtype flags;

int COMsysInit(char *filename, int baud, char parity, int handshake)
{
  int comport=-1;
  int parnum=EVEN_PARITY;
  int hand=0;

  if (!stricmp(filename,"COM1")) comport=COM1;
  if (!stricmp(filename,"COM2")) comport=COM2;
  if (!stricmp(filename,"COM3")) comport=COM3;
  if (!stricmp(filename,"COM4")) comport=COM4;
  if (comport==-1) return FALSE;

  switch (parity) {
    case CASPORT_NONE: parnum=NO_PARITY;break;
    case CASPORT_ODD: parnum=ODD_PARITY;break;
    case CASPORT_EVEN: ;
    default: parnum=EVEN_PARITY;break;
  }

  if (handshake&CASPORT_DTR) hand |=DTR;
  if (handshake&CASPORT_RTS) hand |=RTS;

  /* Initialise COM port */

  if (ASyncInit(comport))
    return FALSE;

  atexit(COMsysClose);	/* cleans up COM port upon exit */

  /* set baud rate and parity.  Calc uses 8 data bits + 2 stop bits */

  ASyncSet(baud,BITS_8|parnum|STOP_2);

  /* raise Data Terminal Ready line to power interface cable */

  if (hand) ASyncHand(hand);

  return TRUE;
}


void COMsysClose(void)
{
  ASyncStop();
}

void COMsysPutChar(unsigned char data)
{
  ASyncOut(data);
}

unsigned char COMsysGetChar(void)
{
  for(;;) {
    if (ASyncInStat())
      return ASyncIn();
    if (kbhit())
      Msg(EXIT_COMM,"comm","User terminated operation");
  }
  /* Msg(EXIT_COMM,"comm/read","Timeout reading from COM port"); */
  return 0;
}

unsigned char COMsysGetCharTimeout(int *success_r,unsigned int modifier)
{
  unsigned long x;
  unsigned long timeout;

  timeout=COMM_TIMEOUT * 100 / modifier;

  for(x=0;x<timeout;x++) {
    if (ASyncInStat()) {
      *success_r=TRUE;
      return ASyncIn();
    }
  }
  *success_r=FALSE;
  return 1;
}

/* end of cturboc.c */
