/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * FFXP.C  Routines for reading/writing FXP files by Bob Parish's program
 */
 
#define _CAS_FFXP_C_

#include "cas.h"
#include "misc.h"
#include "list.h"
#include "ffxp.h"

void FXPWriteData(char *filename, casinfo *info)
{
  FILE *fxp=NULL;
  unsigned char *data=NULL;
  int n;

  for(;info;info=info->next) {
    switch (info->type) {
      case tEditor:
        Msg(M_WARN,"fxp.write","FXP format does not support filenames or passwords");
      case tOldProg:
        if (info->type==tOldProg) {
          if (info->s.prg.progtype!=0)
            Msg(M_WARN,"fxp.write","FXP format does not support program types other than COMP");
          if (info->s.prg.prognum>=0 && info->s.prg.prognum<=37)
            Msg(M_WARN,"fxp.write","FXP format does not support specified program locations");
        }
        fxp=tsys_fopen(filename,"w",FALSE);
        if (!fxp) {
          Msg(M_ERROR,"fxp.write","unable to open `%s'",filename);
          return;
        }
        fprintf(fxp,"%ld\n%d\n",(long)info->length + 2,':');
        data=info->data;
        for (n=0;n<info->length;n++) {
          fprintf(fxp,"%d\n",data[n]);
        }
        fprintf(fxp,"%d\n0\n",CalcCSum(info->data,info->length));
        tsys_fclose(fxp);
        return;

      case tDesc:
      case tText:
        break;

      default:
        Msg(M_WARN,"fxp.write","FXP format does not support blocks of type: %s",
                                    s_types[info->type]);
    }
  }
}


void FXPReadData(char *filename, casinfo *info)
{
  char line[CAS_FXP_LINELENGTH];
  FILE *fxp=NULL;
  unsigned char *data=NULL;
  int n;
  int rcsum,ccsum;
  casinfo *curblk;

  curblk=tsys_malloc(sizeof (casinfo));
  curblk->data=data=tsys_malloc(CAS_MEMSIZE);
  curblk->length=0;
  curblk->type=tOldProg;
  curblk->next=NULL;

  if (!(fxp=tsys_fopen(filename,"r",FALSE))) {
    Msg(M_ERROR,"fxp.read","Unable to open `%s'",filename);
    free(curblk->data);
    free(curblk);
    return;
  }

  fgets(line,CAS_FXP_LINELENGTH,fxp);  /* read size info */
  curblk->length=atoi(line) - 2;
  
  fgets(line,CAS_FXP_LINELENGTH,fxp);  /* eat colon */

  for (n=0;n<curblk->length;n++) {
    fgets(line,CAS_FXP_LINELENGTH,fxp);
    *(data++)=(unsigned) atoi(line) & 0xff;
  }

  fgets(line,CAS_FXP_LINELENGTH,fxp);
  rcsum=atoi(line);
  ccsum=CalcCSum(data,curblk->length);

  if (rcsum!=ccsum) {
    Msg(M_WARN,"fxp.read","Checksum error.  read 0x%02x, expected 0x%02x",
                rcsum,ccsum);
  }

  if (curblk->length <=0) {
    free(curblk->data);
    free(curblk);
  } else {
    info->next=curblk;
    curblk->data=tsys_realloc(data,curblk->length);
  }

  tsys_fclose(fxp);
}


/* end of ffxp.c */
