/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * MISC.H  Miscellaneous utility functions and std library wrappers
 */

#ifndef _CAS_MISC_H_
#define _CAS_MISC_H_

#include "mylibc.h"		/* most of the old functions from misc.[ch] */

/*  +ve values denote FATAL and program error code, see EXIT_* in cas.h  */

#define M_MSG           -1
#define M_WARN          -2
#define M_ERROR         -3
#define M_VERBOSE       -4
#define M_DEBUG         -5

#define PAGER_LINES			20

#define PAGER_CLEAR         0       /* clears line count */
#define PAGER_FORCE         1       /* ignores flags.pager setting */
#define PAGER_ADD           2       /* Standard.  count++, page if >=20 */
#define PAGER_NOW           4       /* Page now, even if count!=20 */
#define PAGER_NOMSG         8       /* Don't display message */
#define PAGER_NOPAUSE       0x10    /* don't page, whatever (just incr) */
#define PAGER_NORESET       0x20    /* Don't reset counter to 0 on page */
#define PAGER_NOADD         0x40    /* don't increment counter */
  
/* prototypes */

/* display functions */
void		Msg		(int type, const char *section, const char *msgstr, ...);
void		Output	(const char *fmt, ...);
void		Pager	(int inc);

/* Enumeration_IO type functions */

formats		string_to_format	(char *str);
types		string_to_type		(char *str);

/* deal with CAS programs which use misc.c but not castle.c (eg tokcheck) */

#ifdef NO_CASTLE
#include "castle.h"
void castle_send(castle_code type, const char *fmt, va_list args);
void castle_send(castle_code type, const char *fmt, va_list args) {/* noop */}
#endif /* NO_CASTLE */

#endif /* _CAS_MISC_H_ */

/* end of misc.h */
