/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * MYLIBC.H  Miscellaneous utility functions and std library wrappers
 *           Originally written for use with CaS.
 */

#ifndef _MYLIBC_H_
#define _MYLIBC_H_

#ifdef _MYLIBC_C_
extern char *argv0;
#endif

#ifndef EXIT_MEM	/* in CaS, this is defined in cas.h */
#define EXIT_MEM			3
#endif

#ifndef TRUE
#define TRUE    1           /* use only for assignment, never comparison */
#endif
#ifndef FALSE
#define FALSE    0
#endif

/* These typedefs work for every compiler I use.  Which means they probably
 * don't for your compiler :-)
 */

/* Stolen from QEU 0.3 */

typedef signed char             Int8;
typedef unsigned char           UInt8;
typedef short int               Int16;
typedef unsigned short int      UInt16;
typedef long                    Int32;
typedef unsigned long           UInt32;
typedef float                   Float32;

/* Turbo C 2.0 doesn't seem to be forthcoming with FILENAME_MAX in stdio.c
 * AFAICR, so here it is defined at a suitable nasty size
 */

#ifndef FILENAME_MAX 
#define FILENAME_MAX        127
#endif

/* prototypes */

unsigned long   Hash      (char *s, unsigned max);
unsigned long   Hash2     (char *s, unsigned max); /* for probing */
int             WildMatch (char *wildcard, char *teststr);
unsigned char   CalcCSum  (void *data, int size);
char            *strdup_trim    (char *s1, char *strip);

/* wrappers for standard library functions */

void    *tsys_malloc      (size_t size);
void    *tsys_calloc      (size_t size);
void    *tsys_realloc     (void *oldptr, size_t size);
FILE    *tsys_fopen       (char *fname, char *faccess, int owndir);
int     tsys_fclose       (FILE *handle);

/* NULL safe strcmp, strncmp, stricmp, strnicmp */

int	tsys_strcmp (const char *s1, const char *s2);
int tsys_strncmp (const char *s1, const char *s2, int n);
int tsys_stricmp (const char * s1, const char * s2);
int tsys_strnicmp (const char *s1, const char *s2, int n);

/* functions that should be defined in the standard library */

char    *tsys_strdup      (char *s1);

/* We often need to use the case insensitive string comparison functions
 * stri[n]cmp.  These may need to be prototyped, or even substituted for
 * the BSD style str[n]casecmp. (Which has a less C like name, imho).
 * With gcc -ansi, these functions are not prototyped, as they are non-ANSI.
 */

#ifdef __DJGPP__    /* Not prototyped when -ansi, but are in libc. */
# ifdef __STRICT_ANSI__
int	stricmp(const char *_s1, const char *_s2);
int	strnicmp(const char *_s1, const char *_s2, size_t _n);
# endif
#endif

#if defined(__linux__) || defined(linux) || defined(__sgi__) || defined(sgi)
# define stricmp(x,y)            strcasecmp((x),(y))
# define strnicmp(x,y,z)         strncasecmp((x),(y),(z))
# ifdef __STRICT_ANSI__		/* we'll just assume they're implemented */
int	strcasecmp(const char *_s1, const char *_s2);
int	strncasecmp(const char *_s1, const char *_s2, size_t _n);
# endif
#endif

/* Stolen from QEU 0.3.  Unused */

#ifdef FAT_ENDIAN
UInt32 SwapInt32	(UInt32 x);
#else
#define SwapInt32(x)	(x)
#endif /* FAT_ENDIAN */

#endif /* _MYLIBC_H_ */

/* end of mylibc.h */
