/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * S9700.H  Routines for translating the 7700, 9700, 9800 .CAS format
 */
 
#ifndef _CAS_S9700_H_
#define _CAS_S9700_H_

#define HDR_SIZE_9700             38

#define SS_ORDER_UNKNOWN    '\0'
#define SS_ORDER_9700       'D'
#define SS_ORDER_9800       'R'
  /* we set 9800 order to be the default, as it is the more logical of
   * the two.
   */
#define SS_ORDER_DEFAULT    SS_ORDER_9800

#define CAS_PZ_NUMPROGS     38      /* Number of programs in PZ file */

void ReadData_9700(void *firsthdr, casinfo *info, iostruct *io);
void WriteData_9700(casinfo *info, iostruct *io);
int Is_9700(unsigned char *data);

#ifdef _CAS_S9700_C_

typedef struct {
  unsigned char magic[2];        /* indicates the file type */
  unsigned char misc[5];         /* miscellaneous data */
  unsigned char filename[12];    /* Editor filename */
  unsigned char password[12];    /* Editor password */
  unsigned char reserved[7];     /* Normally FF FF FF, reserved? */
} casheader;

typedef struct {
  unsigned char unknown1;
  unsigned char sizehi;          /* Big-endian size of program, including */
  unsigned char sizelo;          /* start/stop chars and checksum */
  unsigned char progtype;        /* Program type */
  unsigned char unknown3;
} prgheader;

/* Helper functions */

static casinfo *PrepareInfo(types type, size_t datalen);
static casheader *PrepareHeader(char magic[2], char misc);
static int IsMultiple(casinfo *info);
static int read_ss_block(casinfo *info, iostruct *io);
static int write_ss_block(casinfo *info, unsigned char col, iostruct *io);

/* Main type specific data reading functions */

#define DC(x) static void Read##x (casheader *, casinfo *, iostruct *); \
              static void Write##x (casinfo *, iostruct *)

#undef DC

static void ReadRaw(casheader *hdr, casinfo *info, iostruct *io);
static void ReadP1(casheader *hdr, casinfo *info, iostruct *io);
static void ReadPZ(casheader *hdr, casinfo *info, iostruct *io);
static void ReadEN(casheader *hdr, casinfo *info, iostruct *io);
static void ReadFN(casheader *hdr, casinfo *info, iostruct *io);
static void ReadDD(casheader *hdr, casinfo *info, iostruct *io);
static void ReadDC(casheader *hdr, casinfo *info, iostruct *io);

static void WriteRaw(casinfo *info, iostruct *io);
static void WriteP1(casinfo *info, iostruct *io);
static void WritePZ(casinfo *info, iostruct *io);
static void WriteEN(casinfo *info, iostruct *io);
static void WriteFN(casinfo *info, iostruct *io);
static void WriteDD(casinfo *info, iostruct *io);
static void WriteDC(casinfo *info, iostruct *io);

typedef struct {
  char magic[2];
  types type;
  void (*read)(casheader *, casinfo *, iostruct *);
  void (*write)(casinfo *, iostruct *);
} castypes_t;

static castypes_t castypes[] = {
  {"AA",tDyna,ReadRaw,WriteRaw},
  {"AD",tAllMem,ReadRaw,WriteRaw},
  {"AL",tAll,ReadRaw,WriteRaw},
  {"AM",tVarMem,ReadRaw,WriteRaw},
  {"BU",tBackup,ReadRaw,WriteRaw},
  {"DC",tSSCol,ReadDC,WriteDC},
  {"DD",tSSMono,ReadDD,WriteDD},
  {"DM",tDefMem,ReadRaw,WriteRaw},
  {"EN",tEditor,ReadEN,WriteEN},
  {"FN",tEditor,ReadFN,WriteFN},
  {"F1",tFn,ReadRaw,WriteRaw},
  {"F6",tFn,ReadRaw,WriteRaw},
  {"GA",tGraphs,ReadRaw,WriteRaw},
  {"GF",tZoom,ReadRaw,WriteRaw},
  {"GR",tRange,ReadRaw,WriteRaw},
  {"GT",tFnTab,ReadRaw,WriteRaw},
  {"MA",tMatrix,ReadRaw,WriteRaw},
  {"PD",tPoly,ReadRaw,WriteRaw},
  {"P1",tOldProg,ReadP1,WriteP1},
  {"PZ",tOldProg,ReadPZ,WritePZ},
  {"RT",tRecTab,ReadRaw,WriteRaw},
  {"SD",tSimul,ReadRaw,WriteRaw},
  {"SR",tLR,ReadRaw,WriteRaw},
  {"SS",tSD,ReadRaw,WriteRaw},
  {"\x17\xff",tEnd,ReadRaw,WriteRaw},
  {"::",tUnknown,ReadRaw,WriteRaw}
};


#endif /* _CAS_S9700_C_ */

#endif /* _CAS_S9700_H_ */

/* end of s9700.h */

