/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * S9850.H  Routines for translating the 9750, 9850, 9950 .CAS format
 */
 
#ifndef _CAS_S9850_H_
#define _CAS_S9850_H_

#define HDR_SIZE_9850             48

void ReadData_9850(void *firsthdr, casinfo *info, iostruct *io);
void WriteData_9850(casinfo *info, iostruct *io);
int Is_9850(unsigned char *data);

#ifdef _CAS_S9850_C_

typedef struct {
  unsigned char format[3];        /* General file format */
  unsigned char unknown1;        /* '\0' */
  unsigned char magic[2];        /* indicates the file type */
  unsigned char unknown2;        /* '\0' */
  unsigned char unknown3;        /* '\0' */
  unsigned char sizehi;          /* hi byte of data size */
  unsigned char sizelo;          /* hi byte of data size */
  unsigned char filename[8];     /* Program filename */
  unsigned char unknown4[8];     /* '0xff' */
  unsigned char password[8];     /* Editor password */
  unsigned char n;               /* 'N' */
  unsigned char l;               /* 'L' */
  unsigned char unknown5[12];    /* '0xff' */
} casheader;

/* Helper functions */

static casinfo *PrepareInfo(types type, size_t datalen);
static casheader *PrepareHeader(char format[3],char magic[2]);

/* Main type specific data reading functions */

static int ReadRaw(casheader *hdr, casinfo *info, iostruct *io);
static int ReadPG(casheader *hdr, casinfo *info, iostruct *io);

static int WriteRaw(casinfo *info, iostruct *io);
static int WritePG(casinfo *info, iostruct *io);

typedef struct {
  char format[3];
  char magic[2];
  types type;
  int (*read)(casheader *, casinfo *, iostruct *);
  int (*write)(casinfo *, iostruct *);
} castypes_t;

static castypes_t castypes[] = {
  {"TXT","PG",tEditor,ReadPG,WritePG},
  {"VAL","MT",tMatrix,ReadRaw,WriteRaw},
  {"END","\xff\xff",tEnd,ReadRaw,WriteRaw},
  {":::","\xff\xff",tUnknown,ReadRaw,WriteRaw}
};


#endif /* _CAS_S9850_C_ */

#endif /* _CAS_S9850_H_ */

/* end of s9850.h */

