/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * TOKCHECK.C   Checks CaS .tok token files.
 */

#define CAS_DEFINE_GLOBALS
#define NO_CASTLE
#define _CAS_TOKCHECK_C_
#define CAS_TOKCHECK

#include "cas.h"
#include "misc.h"
#include "token.h"
#include "tokcheck.h"

int main(int argc, char **argv)
{
  int h,i,on,os,ln,ls,nne;
  int hash_usage;
  int ctoken;
  char *argw=NULL;
  char *model="All";

  argv0=argv[0];
  flags.debug=stdout;

  printf("tokcheck, (c) 1996 Tom Wheeley.  A part of CaS %s\n",CAS_VER_STR);

  if (argc >1 && !strcmp(argv[1],"-?")) {
    printf("\nusage: %s [tokfile] [model]\n",*argv);
    exit(EXIT_CMD);
  }

  if (argc>=3) {
    argw=argv[2];
    if (!strnicmp(argw,"cfx",3)) argw+=3;
    if (!strnicmp(argw,"fx",2)) argw+=2;
     if (!strcmp(argw,"7700") || !strcmp(argw,"7")) {
      flags.model=MDL_7700; model="7700"; }
    if (!strcmp(argw,"9700") || !strcmp(argw,"9")) {
      flags.model=MDL_9700; model="9700"; }
    if (!strcmp(argw,"9800") || !strcmp(argw,"8")) {
      flags.model=MDL_9800; model="9800"; }
    if (!strcmp(argw,"9750") ) {
      flags.model=MDL_9750; model="9850"; }
    if (!strcmp(argw,"9850") || !strcmp(argw,"5")) {
      flags.model=MDL_9850; model="9850"; }
    if (!strcmp(argw,"9950") ) {
      flags.model=MDL_9950; model="9850"; }
    if (!stricmp(argw,"all") || !strcmp(argw,"*")) {
      flags.model=MDL_UNKNOWN; model="All"; }
  }

  if (argc>=2)
    argw=argv[1];
  else
    argw="ctf.tok";

  printf("\nToken Check report for `%s'\n",argw);
  for (i=25+strlen(argw);i > 0;--i)
    putchar('=');

  printf("\n\nModel: %s",model);

  printf("\n\nRuntime errors\n"
             "~~~~~~~~~~~~~~\n");

  token_init(argw, argw, argw);

  printf ("\n\nHashing information\n"
              "~~~~~~~~~~~~~~~~~~~");

  i=0;hash_usage=0;
  for (h=0;h<TOKEN_HASHSIZE;h++) {
    if (h==i) {
      printf("\n%5d: ",i);
      i+=TOKCHECK_INTERVAL;
    }
    if (toklis->in[h].token) {
      putchar('#');
      ++hash_usage;
    }
  }

  printf("\n\nOf 0x%04x entries, 0x%04x used (%02.2f%%), 0x%04x empty (%02.2f%%)",
                     TOKEN_HASHSIZE,hash_usage,
                     (float)hash_usage/TOKEN_HASHSIZE*100,
                     TOKEN_HASHSIZE - hash_usage,
                     (float)(TOKEN_HASHSIZE-hash_usage)/TOKEN_HASHSIZE*100);

  printf("\n\nToken count information\n"
             "~~~~~~~~~~~~~~~~~~~~~~~\n");
  printf("ctok [%%ff] i on os ln ls ok? hashcodes token text\n");

  nne=0;
  for(ctoken=1;ctoken<TOKEN_MAX;ctoken++) {
    i=0;on=0;os=0;ln=0;ls=0;
    for (h=0;h<TOKEN_HASHSIZE;h++)
      if ((toklis->in[h].code&TOKEN_MASK)==ctoken) ++i;
    if (toklis->out_nice[ctoken]) ++on;
    if (toklis->out_str[ctoken]) ++os;
    if (toklis->list_nice[ctoken]) ++ln;
    if (toklis->list_str[ctoken]) ++ls;

    if (i+on+os+ln+ls) {    /* if at least one field is set */
      printf("%03d[%03d] %2d (%d %d) (%d %d)",ctoken,ctoken%256,i,on,os,ln,ls);
      if (i==0||on==0||os==0||ln==0||ls==0) printf(" ! "); else printf("   ");

      for (h=0;h<TOKEN_HASHSIZE;h++)
        if ((toklis->in[h].code&TOKEN_MASK)==ctoken) printf("%04x ",h);

      if (os)
        printf("`%s' ",toklis->out_str[ctoken]);
      else if (on)
        printf("`%s' ",toklis->out_nice[ctoken]);
      else if (ls)
        printf("{l}`%s' ",toklis->list_str[ctoken]);
      else if (ln)
        printf("{l}`%s' ",toklis->list_nice[ctoken]);
      else
        printf("`' ");

      if (toklis->text[ctoken]) {
        printf("\"%s\"",toklis->text[ctoken]);
      } else {
        printf("\"\"");
      }

      putchar('\n');
    }
  }

  printf("\n\nMissing tokens\n"
             "~~~~~~~~~~~~~~\n");
  for(ctoken=1;ctoken<TOKEN_MAX;ctoken++) {
    i=0;
    for (h=0;h<TOKEN_HASHSIZE;h++)
      if ((toklis->in[h].code&TOKEN_MASK)==ctoken) ++i;
    if (toklis->out_nice[ctoken]) ++i;
    if (toklis->out_str[ctoken]) ++i;
    if (toklis->list_nice[ctoken]) ++i;
    if (toklis->list_str[ctoken]) ++i;
    if (!i) printf("%03d ",ctoken);
  }

  token_free();

  printf("\n{eof}\n");
  return EXIT_SUCCESS;
}

/* end of tokcheck.c */
