/* Copyright (C) 1997 Tom Wheeley, see file COPYING for details */

/*
 * TOKEN.H  Routines for loading token files, and converting to and from
 *          text tokens and integer ctokens
 */
 
#ifndef _CAS_TOKEN_H_
#define _CAS_TOKEN_H_

#define TOKEN_LINESIZE                   255     /* Max line length in tok  */
#define TOKEN_MIN                       1       /* Minimum ctoken number   */
#define TOKEN_MAX                       767     /* Number of ctokens       */
#define TOKEN_HASHSIZE                  8011    /* hash table (prime)      */
#define TOKEN_HASHSIZE_2                8009    /* hash2, prime            */
#define TOKEN_BLOCKSIZE                 65500   /* size of token block     */
#define PARSE_TOKSIZE                   16      /* maximum length of token */

#define TOKEN_OUT				0
#define TOKEN_LIST				2
#define TOKEN_NICE				0
#define TOKEN_STR				1

#define TOKEN_OUT_NICE		(TOKEN_OUT | TOKEN_NICE)
#define TOKEN_OUT_STR		(TOKEN_OUT | TOKEN_STR)
#define TOKEN_LIST_NICE		(TOKEN_LIST | TOKEN_NICE)
#define TOKEN_LIST_STR		(TOKEN_LIST | TOKEN_STR)

/*
 * The hash table must be large enough to contain every single pointer.
 * Extended clashes should also be avoided, so bigger == better
 * Inifinite loop will result if table too small!  Entries are (short+char*)
 */

#define TF_UNDEFINED            0         /* reads no tokens */
/* flags 0x01 -> 0x80 reserved for models, default allow all */
#define TF_DEFAULT              0x00ffU
#define TF_LISTONLY             0x0100U   /* Only to be used in list mode */
#define TF_TEXT                 0x0200U   /* Token text */
/* Following flags are stored. 0x0400 upwards are free. */
#define TF_FILENAME             0x2000U
#define TF_NNE                  0x4000U   /* not normally enterable */
#define TF_VERBATIM             0x8000U   /* not in strings */
/* perhaps confusingly, VERBATIM tokens are explicitly NOT verbatim
 * copies of the token on the calc. */

/* Both flags and token number are stored in the `code' entry of the hash
   table, for space efficiency.  The flags are ANDed with the FLAG_MASK
   so that low-numbered flags (such as model info) doesn't interfere with
   token number information. */

#define TOKEN_MASK              0x03ffU
#define FLAG_MASK               0xfc00U

typedef struct {
  unsigned short code;              /* must be >= 16 bit, but short */
  char *token;                      /* by definition is >= 16bit    */
} ht_entry;

typedef struct tokenlist {
  ht_entry in[TOKEN_HASHSIZE];      /* hash table */
  char *out_nice[TOKEN_MAX];        /* array of nicest, non-string tokens */
  char *out_str[TOKEN_MAX];         /* array of unambiguous string tokens */
  char *list_nice[TOKEN_MAX];        /* array for nice list tokens */
  char *list_str[TOKEN_MAX];		/* array for string list tokens */

  char *text[TOKEN_MAX];            /* array of token descriptions */

  /* memory management */
  char *tokblock;                   /* start of token text image */
  char *tokpos;                     /* current position in token image */
  char *tokend;                     /* end of token image (safety) */
} tokenlist;

#ifdef _CAS_TOKEN_C_
tokenlist *toklis=NULL;
#else
extern tokenlist *toklis;
#endif /* _CAS_TOKEN_C_ */

/* function prototypes */

void token_init(char * in_tokf, char *out_tokf, char *list_tokf);
void token_free(void);
char *code_to_token(int ctoken, int display);
int token_to_code(char *token, unsigned set, unsigned unset);
char *code_to_text(unsigned char *toks, int display);
unsigned char *text_to_code(char *text, int max);

#ifdef _CAS_TOKEN_C_

void token_load(char *tokfile, int in, int out, int list);
int token_add_line(char *tokline,int in,int out,int list);
int token_get_flags(char **token_r);
char *token_expand(char *token);
unsigned long token_add_to_hash(char *token,int toknum);

#endif /* _CAS_TOKEN_C_ */

#endif /* _CAS_TOKEN_H_ */

/* end of token.h */
